﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Scheduling.MRCPSP {

  [Item(nameof(ResourceDemand), "")]
  [StorableClass]
  public sealed class ResourceDemand : ParameterizedNamedItem {
    //public ResourceDemand()
    //  : this() {
    //}

    [Storable]
    private readonly IFixedValueParameter<IntValue> demandParam;

    [Storable]
    private readonly IFixedValueParameter<IntValue> numberParam;

    public ResourceDemand() {
      Parameters.Add(numberParam = new FixedValueParameter<IntValue>(nameof(Number), new IntValue()));
      Parameters.Add(demandParam = new FixedValueParameter<IntValue>(nameof(Demand), new IntValue()));
    }

    public override string ToString() => $"{Name} (Resource {Number})";

    [StorableConstructor]
    private ResourceDemand(bool deserializing)
      : base(deserializing) { }

    private ResourceDemand(ResourceDemand original, Cloner cloner)
      : base(original, cloner) {
      numberParam = cloner.Clone(original.numberParam);
      demandParam = cloner.Clone(original.demandParam);
    }

    public int Demand {
      get => demandParam.Value.Value;
      set => demandParam.Value.Value = value;
    }

    public int Number {
      get => numberParam.Value.Value;
      set => numberParam.Value.Value = value;
    }

    public override IDeepCloneable Clone(Cloner cloner) => new ResourceDemand(this, cloner);
  }
}
